#define _USE_MATH_DEFINES

#include "stdafx.h"
#include "AFKF.h"

#define _USE_MATH_DEFINES

#include <cmath>





AFKF::AFKF(void)
{
 // Number of eye-tracker untis per screen horizontally and vertically
 image_width_etu = 261;//1920.0; //261;
 image_height_etu = 241;//1200.0; //241;

 // Pixel dimentions of the image  
 image_width_pixels = 1920.0;
 image_height_pixels = 1200.0;

 // Metric dimentions of the image in inches  
 image_width_inches = 20.5;
 image_height_inches = 12.75;

 // The distance between the subject and the center of the screen 
 subject_image_distance_inches = 27.0;

 // Calculating the angles
 // This portion calculates the degrees in the visual angle from the center of the screen to the top 
 double x_degrees_half_screen = 180 * atan(0.5*image_height_inches / subject_image_distance_inches)/M_PI;
 // This portion calculates the degrees in the visual angle from the center to left edge
 double y_degrees_half_screen = 180 * atan(0.5*image_width_inches / subject_image_distance_inches)/M_PI;
  
 // Angle dimention of the image measured in degrees
 image_width_deg = 2 * x_degrees_half_screen;
 image_height_deg = 2 * y_degrees_half_screen;
 
 // pixels in one eye-tracker unit
 pix_eye_track_unit_width = image_width_pixels / image_width_etu;
 pix_eye_track_unit_height = image_height_pixels / image_height_etu;

 // average pixels per eye-tracker units
 pix_eye_track_unit = (pix_eye_track_unit_width + pix_eye_track_unit_height)/2;
 
 // degrees in one eye-tracker unit
 deg_eye_track_unit_width = image_width_deg / image_width_etu;
 deg_eye_track_unit_height = image_height_deg / image_height_etu;
 
 // the number of eye-tracker units per inch
 etu_inch_width = image_width_etu / image_width_inches;
 etu_inch_height = image_height_etu / image_height_inches;
 

 // the amount of horizontal eye-tracker units in one degree (pre-calculated)
 x_etu = image_width_etu / image_width_deg;
 // the amount of vertical eye-tracker units in one degree (pre-calculated)
 y_etu = image_height_etu / image_height_deg;

 //Initialize Kalman Filter
 delta_t = 1.0/60.0;
 
 // initialize transition matrix
 trn_mat[0][0] = 1.0;
 trn_mat[0][1] = delta_t;
 trn_mat[1][0] = 0.0;
 trn_mat[1][1] = 1.0;
 
 // initialize error covariance matrix
 x_P[0][0] = 1.0;
 x_P[0][1] = 0.0; 
 x_P[1][0] = 0.0;
 x_P[1][1] = 1.0;

 //Initializtion of the process noise covariance matrix
 Q[0][0] = 1.0;
 Q[0][1] = 0.0; 
 Q[1][0] = 0.0;
 Q[1][1] = 1.0;

 //Initialization of the measurement noise covariance matrix
 R = 1;

 //Initialization of the observation matrix
 H[0] = 1.0;
 H[1] = 0.0;

 //Initialization of Identity matrix
 I = 1.0;
 


 // initialize initial visual systems state
 // initial horizontal eye-position - center of the screen
 x_sys_state_deg[0] = 20.0;
 // initial horizontal eye-velcoity 
 x_sys_state_deg[1] = 0.0;

 // initial vertical eye-position - center of the screen
 y_sys_state_deg[0] = 10.0;
 // initial vertical eye-velcoity 
 y_sys_state_deg[1] = 0.0;

 //initialize the measurement horizontal position
 x_pos_measurement_deg = 0.0;

 //initialize the measurement vertical position
 y_pos_measurement_deg = 0.0;

// Matrix_Mult(trn_mat, P, T);

 // matrix for temporary calculations
// double P[2][2]

}

AFKF::~AFKF(void)
{
}

double AFKF::Current_Position_ETU_X(double x_pos_measurement_etu)
{
	x_pos_measurement_deg = x_pos_measurement_etu * deg_eye_track_unit_width;

	//returns the position translated to pixels
	return x_pos_measurement_etu*pix_eye_track_unit_width; 

}

double AFKF::Current_Position_ETU_Y(double y_pos_measurement_etu)
{
	y_pos_measurement_deg = y_pos_measurement_etu * deg_eye_track_unit_height;

	//returns the position translated to pixels
	return y_pos_measurement_etu * pix_eye_track_unit_height; 

}

double AFKF::Report_KF_Updated_Position_PIX_X()
{
	//Convert from degrees to pixels
	return x_sys_state_deg[0] * x_etu * pix_eye_track_unit_width;

}

double AFKF::Report_KF_Updated_Position_PIX_Y()
{
	//Convert from degrees to pixels
	return y_sys_state_deg[0] * y_etu * pix_eye_track_unit_height;
}

double AFKF::Report_KF_Updated_Position_DEG_X()
{
	return x_sys_state_deg[0];

}

double AFKF::Report_KF_Updated_Position_DEG_Y()
{
	return y_sys_state_deg[0];
}


void AFKF::Kalman_Filter_Mod_X(/*double delta_t sampling interval*/)
{

	// PREDICT PHASE

	//precition of the system's state one step ahead
	double x_system_state_kf_predicted[2];
    Matrix_Mult_22by12(trn_mat, x_sys_state_deg, x_system_state_kf_predicted);

	//projection of the covariance matrix one step ahead 
	double ErrCovMatTemp1[2][2];
	Matrix_Mult_22by22(trn_mat, x_P, ErrCovMatTemp1);
    double trn_mat_transposed[2][2];
    Matrix_Transpose_22(trn_mat, trn_mat_transposed);
	double ErrCovMatTemp2[2][2];
	Matrix_Mult_22by22(ErrCovMatTemp1, trn_mat_transposed, ErrCovMatTemp2);
    double x_ErrCovMat_kf_predicted[2][2];
	Matrix_Add_22by22(ErrCovMatTemp2, Q, x_ErrCovMat_kf_predicted);
    
	//UPDATE PHASE
	//Compute Kalman gain
	double KalmGainTemp1[2];
	Matrix_Mult_22by12(x_ErrCovMat_kf_predicted, H, KalmGainTemp1);
	double KalmGainTemp2 = Matrix_Mult_21by12(H, KalmGainTemp1) + R;
	double x_KalmGain[2];
    Matrix_Divided_By_Number_12by1(KalmGainTemp1, KalmGainTemp2, x_KalmGain);

	//update the estimate of the state vectro with a measurement from the eye-tracker
	double system_state_temp1 = x_pos_measurement_deg - Matrix_Mult_21by12(H, x_system_state_kf_predicted);

	double system_state_temp2[2];
    Matrix_Mult_By_Number_12by1(x_KalmGain, system_state_temp1, system_state_temp2);

	double x_system_state_kf_updated_deg[2];
	Matrix_Add_12by12(x_system_state_kf_predicted, system_state_temp2, x_system_state_kf_updated_deg);

	double temp_state = x_system_state_kf_updated_deg[0];
    //update error covariance matrix

	x_sys_state_deg[0] = x_system_state_kf_updated_deg[0];
	x_sys_state_deg[1] = x_system_state_kf_updated_deg[1];

    double ErrCovMatTemp3 = I - Matrix_Mult_21by12(x_KalmGain, H);
	Matrix_Mult_By_Number_22by1(x_ErrCovMat_kf_predicted, ErrCovMatTemp3, x_P);



}


void AFKF::Kalman_Filter_Mod_Y(/*double delta_t sampling interval*/)
{

	// PREDICT PHASE

	//precition of the system's state one step ahead
	double y_system_state_kf_predicted[2];
    Matrix_Mult_22by12(trn_mat, y_sys_state_deg, y_system_state_kf_predicted);

	//projection of the covariance matrix one step ahead 
	double ErrCovMatTemp1[2][2];
	Matrix_Mult_22by22(trn_mat, y_P, ErrCovMatTemp1);
    double trn_mat_transposed[2][2];
    Matrix_Transpose_22(trn_mat, trn_mat_transposed);
	double ErrCovMatTemp2[2][2];
	Matrix_Mult_22by22(ErrCovMatTemp1, trn_mat_transposed, ErrCovMatTemp2);
    double y_ErrCovMat_kf_predicted[2][2];
	Matrix_Add_22by22(ErrCovMatTemp2, Q, y_ErrCovMat_kf_predicted);
    
	//UPDATE PHASE
	//Compute Kalman gain
	double KalmGainTemp1[2];
	Matrix_Mult_22by12(y_ErrCovMat_kf_predicted, H, KalmGainTemp1);
	double KalmGainTemp2 = Matrix_Mult_21by12(H, KalmGainTemp1) + R;
	double y_KalmGain[2];
    Matrix_Divided_By_Number_12by1(KalmGainTemp1, KalmGainTemp2, y_KalmGain);

	//update the estimate of the state vectro with a measurement from the eye-tracker
	double system_state_temp1 = y_pos_measurement_deg - Matrix_Mult_21by12(H, y_system_state_kf_predicted);

	double system_state_temp2[2];
    Matrix_Mult_By_Number_12by1(y_KalmGain, system_state_temp1, system_state_temp2);

	double y_system_state_kf_updated_deg[2];
	Matrix_Add_12by12(y_system_state_kf_predicted, system_state_temp2, y_system_state_kf_updated_deg);

	y_sys_state_deg[0] = y_system_state_kf_updated_deg[0];
	y_sys_state_deg[1] = y_system_state_kf_updated_deg[1];

    //update error covariance matrix

    double ErrCovMatTemp3 = I - Matrix_Mult_21by12(y_KalmGain, H);
	Matrix_Mult_By_Number_22by1(y_ErrCovMat_kf_predicted, ErrCovMatTemp3, y_P);



}



//Implementation of two 2 by 2 matrix multipliction
void AFKF::Matrix_Mult_22by22(double A[2][2], double B[2][2], double (&Temp)[2][2])
{
	Temp[0][0] = A[0][0]*B[0][0] + A[0][1]*B[1][0];
	Temp[0][1] = A[0][0]*B[0][1] + A[0][1]*B[1][1];
	Temp[1][0] = A[1][0]*B[0][0] + A[1][1]*B[1][0];
	Temp[1][1] = A[1][0]*B[0][1] + A[1][1]*B[1][1];
}

void AFKF::Matrix_Mult_22by12(double A[2][2], double B[2], double (&Temp)[2])
{
	Temp[0] = A[0][0]*B[0] + A[0][1]*B[1];
	Temp[1] = A[1][0]*B[0] + A[1][1]*B[1];
}

double AFKF::Matrix_Mult_21by12(double A[2], double B[2])
{
	return A[0]*B[0] + A[1]*B[1];
}

void AFKF::Matrix_Mult_By_Number_12by1(double A[2], double B, double (&Temp)[2])
{
	Temp[0] = A[0] * B;
	Temp[1] = A[1] * B;
}

void AFKF::Matrix_Mult_By_Number_22by1(double A[2][2], double B, double (&Temp)[2][2])
{
	Temp[0][0] = A[0][0] * B;
	Temp[0][1] = A[0][1] * B;
	Temp[1][0] = A[1][0] * B;
	Temp[1][1] = A[1][1] * B;
	
}

void AFKF::Matrix_Add_22by22(double A[2][2], double B[2][2], double (&Temp)[2][2])
{
	Temp[0][0] = A[0][0] + B[0][0];
	Temp[0][1] = A[0][1] + B[0][1];
	Temp[1][0] = A[1][0] + B[1][0];
	Temp[1][1] = A[1][1] + B[1][1];
}

void AFKF::Matrix_Add_12by12(double A[2], double B[2], double (&Temp)[2])
{
	Temp[0] = A[0] + B[0];
	Temp[1] = A[1] + B[1];
}

void AFKF::Matrix_Transpose_22(double Original[2][2], double (&Transpose)[2][2])
{

	Transpose[0][0] = Original[0][0];
	Transpose[0][1] = Original[1][0];
	Transpose[1][0] = Original[0][1];
	Transpose[1][1] = Original[1][1];

}



void AFKF::Matrix_Divided_By_Number_12by1(double A[2], double B, double (&Temp)[2])
{
	Temp[0] = A[0] / B;
	Temp[1] = A[1] / B;
}

