/* 
   This Kalman Filter class is going to be updated through the
   Kalman_Filter_Mod(...) function.
   Each time this function is called the position and velocity values
   are going to be predicted and then updated
*/


#pragma once

// #if !defined _USE_MATH_DEFINES
// #define

class AFKF
{
public:
	AFKF(void);


	//sampling interval
	double delta_t; 
    
	//This function performs Kalman Filter calculations for horizontal componenets
	void Kalman_Filter_Mod_X(/*double delta_t*/);

	//This function performs Kalman Filter calculations for vertical components
	void Kalman_Filter_Mod_Y(/*double delta_t*/);
   
	//This function coverts the eye-postion reported in eye-tracker units (etu) to the eye position
	// measured in degrees for horizontal component
	//returns the position translated to pixels
	double Current_Position_ETU_X(double position_measurement_etu);

	// -//- for vertical component
	//returns the position translated to pixels
	double Current_Position_ETU_Y(double position_measurement_etu);

	//This function will return the updated eye position in pixels to the caller for horizontal component
	double Report_KF_Updated_Position_PIX_X();

	//This function will return the updated eye position in pixels to the caller for vertical component
	double Report_KF_Updated_Position_PIX_Y();

	//This function will return the updated eye position in degrees to the caller for horizontal component
	double Report_KF_Updated_Position_DEG_X();

	//This function will return the updated eye position in degrees to the caller for vertical component
	double Report_KF_Updated_Position_DEG_Y();

	//multiply 2 matrixes - digits in definition represent dimentions
	void Matrix_Mult_22by22(double A[2][2], double B[2][2], double (&Temp)[2][2]);

	void Matrix_Mult_22by12(double A[2][2], double B[2], double (&Temp)[2]);

	double Matrix_Mult_21by12(double A[2], double B[2]);

	//add 2 matrixes
	void Matrix_Add_22by22(double A[2][2], double B[2][2], double (&Temp)[2][2]);

	void Matrix_Add_12by12(double A[2], double B[2], double (&Temp)[2]);

	// Transpose a matrix
	void Matrix_Transpose_22(double Original[2][2], double (&Transpose)[2][2]);

	// Divide 1 by 2 matrix by a number B
	void Matrix_Divided_By_Number_12by1(double A[2], double B, double (&Temp)[2]);

	// Multiply 1 by 2 matrix by a number B
	void Matrix_Mult_By_Number_12by1(double A[2], double B, double (&Temp)[2]);
	
	// Multiply 2 by 2 matrix by a number B
	void Matrix_Mult_By_Number_22by1(double A[2][2], double B, double (&Temp)[2][2]);

	//observation (measurement received from the eye-tracker about eye-position) measured in 
	// the eye_tracker units
	double x_pos_measurement_etu;
	double y_pos_measurement_etu;

	//position measurement measured in degrees
    double x_pos_measurement_deg;
    double y_pos_measurement_deg;

    //system state vector sys_state[0] - position, sys_state[0] - velocity
	//system's state vector is vertical vector, system's state measured in degrees
	double x_sys_state_deg[2];
	double y_sys_state_deg[2];

	//Observation matrix - observation matrix is a horizontal vector
	double H[2];

	//transition matrix
	double trn_mat[2][2];

	//error covariance matrix horizontal
	double x_P[2][2];

	//error covariance matrix vertical
	double y_P[2][2];

	//prosses noise covariance matrix
	double Q[2][2];

	//measurement noise covariance matrix
	double R;

	//Identity matrix -  1 dimentional in our case
	double I;


	// Number of eye-tracker untis per screen horizontally and vertically
	double image_width_etu;
    double image_height_etu;

	// Pixel dimentions of the image  
    double image_width_pixels;
    double image_height_pixels;

	// Metric dimentions of the image in inches  
    double image_width_inches;
    double image_height_inches;

	// The distance between the subject and the center of the screen 
    double subject_image_distance_inches;

	// Angle dimention of the image measured in degrees
    double image_width_deg;
    double image_height_deg;

	// pixels in one eye-tracker unit
    double pix_eye_track_unit_width;
    double pix_eye_track_unit_height;

	// average pixels per eye-tracker units
    double pix_eye_track_unit;

	// degrees in one eye-tracker unit
    double deg_eye_track_unit_width;
    double deg_eye_track_unit_height;
 
    // the number of eye-tracker units per inch
    double etu_inch_width;
    double etu_inch_height;
 
    // the amount of horizontal eye-tracker units in one degree (pre-calculated)
    double x_etu;
    // the amount of vertical eye-tracker units in one degree (pre-calculated)
    double y_etu;


public:
	~AFKF(void);
};