// ETSerialPortViewerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "EventHandler.h"
#include "ETSerialPortViewer.h"
#include "ETSerialPortViewerDlg.h"
#include <comdef.h>
#include <Windows.h>
#include <string.h>
#include <time.h>
#include "AFKF.h"
#include "EyeFixationTrigger.h"
#include <deque>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Global data

static const CString REG_SECTION = "Eye Tracker Serial Port Reader";
IASLSerialOutPort3* gpISerialOutPort;

//WOWP begin
void CETSerialPortViewerDlg::Press_Alt_Tab()
{
  keybd_event( VK_MENU,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | 0,
                      0 );
  keybd_event( VK_TAB,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | 0,
                      0 );

  // Simulate a key release
  keybd_event( VK_MENU,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,
                      0);
  keybd_event( VK_TAB,
                      0x45,
                      KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP,
                      0);
}


void CETSerialPortViewerDlg::Left_Mouse_Click_Resease()
{
  INPUT    Input={0};
  // left down 
  Input.type      = INPUT_MOUSE;
  Input.mi.dwFlags  = MOUSEEVENTF_LEFTDOWN;
  ::SendInput(1,&Input,sizeof(INPUT));

  // left up
  ::ZeroMemory(&Input,sizeof(INPUT));
  Input.type      = INPUT_MOUSE;
  Input.mi.dwFlags  = MOUSEEVENTF_LEFTUP;
  ::SendInput(1,&Input,sizeof(INPUT));
}
void CETSerialPortViewerDlg::Mouse_Move_Abs(double x_mov_pix, double y_mov_pix)
{
  double fScreenWidth    = ::GetSystemMetrics( SM_CXSCREEN )-1; 
  double fScreenHeight  = ::GetSystemMetrics( SM_CYSCREEN )-1; 
  double fx = x_mov_pix*(65535.0f/fScreenWidth);
  double fy = y_mov_pix*(65535.0f/fScreenHeight);
  INPUT  Input={0};
  Input.type      = INPUT_MOUSE;
  Input.mi.dwFlags  = MOUSEEVENTF_MOVE|MOUSEEVENTF_ABSOLUTE;
  Input.mi.dx = fx;
  Input.mi.dy = fy;
  ::SendInput(1,&Input,sizeof(INPUT));
}

void CETSerialPortViewerDlg::AddDataRecord(LPSAFEARRAY data, long count)
{

	// Display the message recieved from serial port
	CListCtrl* list = (CListCtrl*)GetDlgItem(IDC_LIST1);

	// limit number of records
	if (list->GetItemCount() > 1000)
		list->DeleteAllItems();

	CComVariant value;
	for (long i=0; i < count; i++)
	{
		// Print value
		SafeArrayGetElement(data, &i, &value);
		value.ChangeType(VT_BSTR);
		CString str = value.bstrVal;
		VariantClear(&value); // prevent memory leak
		if (i==0)
		{
			list->InsertItem (0, str, 0);
		}
		else
		{
			list->SetItemText (0, i, str);
		}
	}

	UpdateData(FALSE);

}


UINT MyThreadProc( LPVOID lpArg )
{

    CETSerialPortViewerDlg * dlg = (CETSerialPortViewerDlg *)lpArg;
    dlg->Display_Msg("Inside of the WoW thread");

	LPSAFEARRAY items;
	long count;
	VARIANT_BOOL bAvailable;
	HRESULT hr;	
	double x_pos_etu, y_pos_etu;
    dlg->kill_wowthread = true;
	clock_t start, finish;
	double duration=0;

    EyeFixationTrigger* eft_left_mouse_click_trigger = new EyeFixationTrigger(1000);



	//Create an instance of the object, which will "click" the mouse when
	//eye-fixation expectations are met.

	//mouse debuggin
	POINT ptCursorPos;
    GetCursorPos(&ptCursorPos);
	double a = ptCursorPos.x;
	double b = ptCursorPos.y;

	CString strMessage;

	// creating new instance of the Kalman Filter
	AFKF* AttFoKaFl = new AFKF();


	int read_counter = 0;

	start = clock();


	while(dlg->kill_wowthread)
	{
	 
        read_counter++; 
		
		// reading eye-position data from the serial port
		hr = gpISerialOutPort->GetScaledData(&items, &count, &bAvailable);
        		
		//mouse simulation
//        GetCursorPos(&ptCursorPos);

		if (FAILED(hr))
		{
			CComBSTR bsError;
			gpISerialOutPort->GetLastError(&bsError);
			CString strError = bsError;
			dlg->Display_Msg(" WoW thread error reading data");
		}

	 	if (bAvailable == VARIANT_TRUE)
		{
      	  dlg->RetrieveXYPosData(items, count, x_pos_etu, y_pos_etu);

		  double x_pos_kf_input_pix = AttFoKaFl->Current_Position_ETU_X(x_pos_etu);
//		  AttFoKaFl->Current_Position_ETU_X(ptCursorPos.x);
		  AttFoKaFl->Kalman_Filter_Mod_X();
		  double x_pos_kf_updated_pix = AttFoKaFl->Report_KF_Updated_Position_PIX_X();
		  double x_pos_kf_updated_deg = AttFoKaFl->Report_KF_Updated_Position_DEG_X();






		  double y_pos_kf_input_pix = AttFoKaFl->Current_Position_ETU_Y(y_pos_etu);
//		  AttFoKaFl->Current_Position_ETU_Y(ptCursorPos.y);
		  AttFoKaFl->Kalman_Filter_Mod_Y();
		  double y_pos_kf_updated_pix = AttFoKaFl->Report_KF_Updated_Position_PIX_Y();
		  double y_pos_kf_updated_deg = AttFoKaFl->Report_KF_Updated_Position_DEG_Y();

		  if(read_counter%25 == 0)
		  {
		    //Report AFKF updated eye-position in pixels
			strMessage.Format("AFKF - input: %.0f hor pos;   %.0f vert pos",x_pos_kf_input_pix, y_pos_kf_input_pix);
			dlg->Display_Msg(strMessage);
			strMessage.Format("AFKF - result: %s hor pos;   %s vert pos",dlg->Double_to_CString_Custom(x_pos_kf_updated_pix), dlg->Double_to_CString_Custom(y_pos_kf_updated_pix));
			dlg->Display_Msg(strMessage);
//			strMessage.Format("MOUSE: %s hor pos;   %s vert pos",dlg->Double_to_CString_Custom(ptCursorPos.x), dlg->Double_to_CString_Custom(ptCursorPos.y));
//            dlg->Display_Msg(strMessage);
		  }


          if(eft_left_mouse_click_trigger->Add_Eye_Pos(x_pos_kf_updated_deg))
		  {
    	     strMessage.Format("FIX DET: %.1f hor pos;   %.1f vert pos",x_pos_kf_updated_pix, y_pos_kf_updated_pix);
			 dlg->Display_Msg(strMessage);
			 dlg->Mouse_Move_Abs(x_pos_kf_updated_pix, y_pos_kf_updated_pix);
			 dlg->Left_Mouse_Click_Resease();
		  }
		  else
		  {
//		  	 dlg->Display_Msg("No fixation");
		  }

		  // Clean up
		  SafeArrayDestroy(items);
	    }
	}

	finish = clock();

	duration = (double)(finish - start) / CLOCKS_PER_SEC;

	double reads_per_sec = read_counter / duration;

	strMessage.Format("%s reads per second",dlg->Double_to_CString_Custom(reads_per_sec )); 
    
    dlg->Display_Msg(strMessage);


    delete eft_left_mouse_click_trigger;
	delete AttFoKaFl;
    return 0;   // thread completed successfully
}


static DWORD __stdcall KalmanFilter( LPVOID lpArg )
{
   ::CoInitialize(NULL);

	LPSAFEARRAY items;
	long count;
	VARIANT_BOOL bAvailable;

	HRESULT hr; 
	
	while(true)
	{
	hr = gpISerialOutPort->GetScaledData(&items, &count, &bAvailable);

	if (FAILED(hr))
	{
		CComBSTR bsError;
		gpISerialOutPort->GetLastError(&bsError);
		CString strError = bsError;
//		MessageBox(strError, "Error reading data", MB_OK | MB_ICONERROR);
		return 0;
	}

	if (bAvailable == VARIANT_TRUE)
	{
		 // Display data record
//		 AddDataRecord(items, count);
		 // Clean up
		 SafeArrayDestroy(items);
	}
	} 

/*
		  CWnd *pWndAct;
		  CRect rect;

		  // Open World of Warcraft window
		  pWndAct = CWnd::FindWindow(NULL,"World of Warcraft");
		  pWndAct->GetClientRect(&rect);
		  pWndAct->SetForegroundWindow();
		  printf("Width = %d, Height = %d\n", rect.Width(), rect.Height());

		  Sleep(500);

		  Left_Mouse_Click_Resease();
		  // Mouse_Move(0, -100);
		  Sleep(500);
		  Mouse_Move_Abs(100, 100);
		 // Mouse_Move_Abs(rect.Width()/2, rect.Height()/2);
		  Sleep(500);
		  Left_Mouse_Click_Resease();
		  Sleep(500); 
		  Mouse_Move_Abs(rect.Width()/2,rect.Height()/4);
		  Sleep(500);
		  Left_Mouse_Click_Resease();
		  Mouse_Move_Abs(rect.Width(), rect.Height());
		  Sleep(500);
		  Left_Mouse_Click_Resease();
		  int i = 0;
*/
   return 0;
}

//WOWP end


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CETSerialPortViewerDlg dialog

CETSerialPortViewerDlg::CETSerialPortViewerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CETSerialPortViewerDlg::IDD, pParent)
	, temp(0)
{
	//{{AFX_DATA_INIT(CETSerialPortViewerDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	gpISerialOutPort = NULL;
	m_pHandler = NULL;
	m_bContinuous = false;
}

void CETSerialPortViewerDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CETSerialPortViewerDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CETSerialPortViewerDlg, CDialog)
	//{{AFX_MSG_MAP(CETSerialPortViewerDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_CONNECT, OnConnect)
	ON_BN_CLICKED(IDC_DISCONNECT, OnDisconnect)
	ON_BN_CLICKED(IDC_GET_DATA, OnGetData)
	ON_BN_CLICKED(IDC_START_CONTINUOUS, OnStartContinuous)
	ON_BN_CLICKED(IDC_STOP_CONTINUOUS, OnStopContinuous)
	ON_BN_CLICKED(IDC_SELECT_CFG_FILE, OnSelectCfgFile)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_WM_CHARTOITEM()
	ON_BN_CLICKED(IDC_FORMAT_STANDARD, OnFormatStandard)
	ON_BN_CLICKED(IDC_FORMAT_FROM_CFG_FILE, OnFormatFromCfgFile)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_NEW_MSG, OnNewMsg)
	ON_BN_CLICKED(IDC_WOWStart, &CETSerialPortViewerDlg::OnBnClickedWowstart)
	ON_BN_CLICKED(IDC_BTEST, &CETSerialPortViewerDlg::OnBnClickedBtest)
	ON_BN_CLICKED(IDC_WOWKILL, &CETSerialPortViewerDlg::OnBnClickedWowkill)
	ON_BN_CLICKED(IDC_ClearWoWStatus, &CETSerialPortViewerDlg::OnBnClickedClearwowstatus)
	ON_BN_CLICKED(IDC_BUTTON1, &CETSerialPortViewerDlg::OnBnClickedButton1)
	ON_BN_CLICKED(IDC_BUTTON2, &CETSerialPortViewerDlg::OnBnClickedButton2)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CETSerialPortViewerDlg message handlers

BOOL CETSerialPortViewerDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	// Get initial settings from the registry
	CWinApp* pApp = AfxGetApp();

	int port = pApp->GetProfileInt(REG_SECTION, "ComPortSel", 0);  // default is COM1
	((CComboBox*)GetDlgItem(IDC_COM_PORT))->SetCurSel(port);

	int ehi = pApp->GetProfileInt(REG_SECTION, "EyeHeadIntegration", 0);
	CheckDlgButton(IDC_EYEHEAD, ehi);
	
	int format = pApp->GetProfileInt(REG_SECTION, "InterfaceSelection", 0);
	CheckRadioButton(IDC_FORMAT_STANDARD, IDC_FORMAT_FROM_CFG_FILE,
		format==1 ? IDC_FORMAT_STANDARD : IDC_FORMAT_FROM_CFG_FILE);

	int streaming = pApp->GetProfileInt(REG_SECTION, "StreamingMode", 0);
	CheckDlgButton(IDC_STREAMING, streaming);

	CString filename = pApp->GetProfileString(REG_SECTION, "CfgFile", "");
	GetDlgItem(IDC_CFG_FILE)->SetWindowText(filename);

	int updateRate = pApp->GetProfileInt(REG_SECTION, "UpdateRate", 1);
	((CComboBox*)GetDlgItem(IDC_UPDATE_RATE))->SetCurSel(updateRate);

	// Baud rate is not selectable by user
    SetDlgItemInt(IDC_BAUD_RATE, 57600);

	// Create COM object
	HRESULT hr = CoCreateInstance(CLSID_ASLSerialOutPort3, NULL, CLSCTX_INPROC_SERVER,
								IID_IASLSerialOutPort3, (void**)&gpISerialOutPort);
	if (FAILED(hr))
	{
		MessageBox("Error creating COM Server ASLSerialOutLib2", "Error", MB_OK | MB_ICONERROR);
		PostMessage(WM_CLOSE);
		return FALSE;
	}

	// Enable appropriate buttons
	EnableDisable(false /*connected*/, false /*continuous reading*/);

    //WOWP begin
	//WoWPercept begin
	listwowpercept = (CListCtrl*)GetDlgItem(IDC_WOWPLIST);
	listwowpercept->InsertColumn (0, "Messages", LVCFMT_LEFT, 400);
	//WoWPercept end
/*
	CRect rect;
	m_cListCtrlWoWPercept.GetClientRect(&rect);
	int nColInterval = rect.Width()/5;
	m_cListCtrlWoWPercept.InsertColumn(0, _T("Item Name"), LVCFMT_LEFT, nColInterval*3);
	m_cListCtrlWoWPercept.InsertColumn(1, _T("Value"), LVCFMT_LEFT, nColInterval);
	m_cListCtrlWoWPercept.InsertColumn(2, _T("Time"), LVCFMT_LEFT, rect.Width()-4*nColInterval);
*/
	//WOWP end

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CETSerialPortViewerDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CETSerialPortViewerDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CETSerialPortViewerDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


void CETSerialPortViewerDlg::OnClose() 
{
	if (m_bContinuous)
	{
		// We cannot exit if continuous reading is active. Also, because of the race condition, 
		// we cannot stop continuous mode, and disconnect inside the same function - 
		// the last call back(s) must be completed in between.
		// Therefore we just stop continuous mode and return.
		// User will have to click "X" one more time to exit.
		OnStopContinuous();
		return;
	}


	// Save user settings in the registry
	CWinApp* pApp = AfxGetApp();
	
	int port = ((CComboBox*)GetDlgItem(IDC_COM_PORT))->GetCurSel();
	pApp->WriteProfileInt(REG_SECTION, "ComPortSel", port);

	int ehi = IsDlgButtonChecked(IDC_EYEHEAD);
	pApp->WriteProfileInt(REG_SECTION, "EyeHeadIntegration", ehi);
	
	int format = IsDlgButtonChecked(IDC_FORMAT_STANDARD);
	pApp->WriteProfileInt(REG_SECTION, "InterfaceSelection", format);

	int streaming = IsDlgButtonChecked(IDC_STREAMING);
	pApp->WriteProfileInt(REG_SECTION, "StreamingMode", streaming);

    CString filename;
    GetDlgItem(IDC_CFG_FILE)->GetWindowText(filename);
	pApp->WriteProfileString(REG_SECTION, "CfgFile", filename);

	int updateRate = ((CComboBox*)GetDlgItem(IDC_UPDATE_RATE))->GetCurSel();
	pApp->WriteProfileInt(REG_SECTION, "UpdateRate", updateRate);


	// Close port connection
	OnDisconnect();

	// Release COM interface
	if (gpISerialOutPort)
		gpISerialOutPort->Release();

	CDialog::OnClose();
}

void CETSerialPortViewerDlg::OnFormatStandard() 
{
	// Enable appropriate buttons
	EnableDisable(false /*connected*/, false /*continuous reading*/);
}

void CETSerialPortViewerDlg::OnFormatFromCfgFile() 
{
	// Enable appropriate buttons
	EnableDisable(false /*connected*/, false /*continuous reading*/);
}

void CETSerialPortViewerDlg::OnConnect() 
{

	// Connect
	CString filename;
	if (IsDlgButtonChecked(IDC_FORMAT_STANDARD))
	{
		// Create temporary configuration file
		char path[200];
		GetCurrentDirectory(175, path);
		strcat(path, "\\ETSerialPortViewer.ini");
		filename = path;

		const char* SECTION = "System Settings";

		char* strFormat = IsDlgButtonChecked(IDC_STREAMING) ? "129" : "1";
		CString strUpdateRate, strBaudRate;
		GetDlgItem(IDC_UPDATE_RATE)->GetWindowText(strUpdateRate);
		GetDlgItem(IDC_BAUD_RATE)->GetWindowText(strBaudRate);

		WritePrivateProfileString (SECTION, "using_6000_serial_out_format", "0", filename);
		WritePrivateProfileString (SECTION, "serial_data_output_format_type", strFormat, filename);
		WritePrivateProfileString (SECTION, "eye_camera_update_rate", strUpdateRate, filename);
		WritePrivateProfileString (SECTION, "serial_out_baud_rate", strBaudRate, filename);

	}
	else // read configuration file provided by user
	{
		GetDlgItem(IDC_CFG_FILE)->GetWindowText(filename);
	}

	// Get user settings
	long comPort = ((CComboBox*)GetDlgItem(IDC_COM_PORT))->GetCurSel() + 1;
	CComBSTR bstrFile = filename;
	VARIANT_BOOL eyeheadIntegration = IsDlgButtonChecked(IDC_EYEHEAD) ? VARIANT_TRUE : VARIANT_FALSE;
	long baudRate, updateRate, itemCount;
	VARIANT_BOOL streamingMode;
	LPSAFEARRAY itemNames;
		
	// Connect to serial out port

	HRESULT hr = gpISerialOutPort->Connect(bstrFile, comPort, eyeheadIntegration,
				&baudRate, &updateRate, &streamingMode, &itemCount, &itemNames);
			
	// update user interface
	if (SUCCEEDED(hr))
	{
		CString strTitle = _T ("ETSerialPortViewer - CONNECTED");
		AfxGetMainWnd ()->SetWindowText (strTitle);
	}
	else
	{
		CString strTitle = _T ("ETSerialPortViewer - FAILED TO CONNECT");
		AfxGetMainWnd ()->SetWindowText (strTitle);
		CComBSTR bsError;
		gpISerialOutPort->GetLastError(&bsError);
		CString strError = bsError;
		MessageBox(strError, "Error connecting to serial port", MB_OK | MB_ICONERROR);
		return;
	}

	// Display values
	SetDlgItemInt(IDC_BAUD_RATE, baudRate);
	CheckDlgButton(IDC_STREAMING, (streamingMode == VARIANT_TRUE));

	char strUpdateRate[15];
	itoa(updateRate, strUpdateRate, 10);
	((CComboBox*)GetDlgItem(IDC_UPDATE_RATE))->SelectString(0, strUpdateRate);
	
	EnableDisable(true /*connected*/, false /*continuous reading*/);
		
	// Prepare results table
	CListCtrl* list = (CListCtrl*)GetDlgItem(IDC_LIST1);
	long ix[1];     // array index must be declared as one-dimentional array
	ix[0] = 0;
	for (long i=0; i < itemCount; i++)
	{
		CComBSTR bstrName;
		SafeArrayGetElement(itemNames, ix, &bstrName);
		CString strName = bstrName; // convert Unicode to ASCII
		int width = strName.GetLength() * 7;
		if (width < 64) width = 64;
		list->InsertColumn (i, strName, LVCFMT_LEFT, width);			
		ix[0]++;
	}
		
	// Clean up
	SafeArrayDestroy(itemNames);
}

void CETSerialPortViewerDlg::OnDisconnect() 
{
	OnStopContinuous();

	if (m_pHandler)
	{
		CComPtr<IUnknown> pUnk = gpISerialOutPort;
		m_pHandler->DispEventUnadvise(pUnk);
		delete m_pHandler;
		m_pHandler = NULL;
	}
	
	// initiate COM object
	if (gpISerialOutPort == NULL)
		return;

	HRESULT hr = gpISerialOutPort->Disconnect();

	// update user interface
	if (SUCCEEDED(hr))
	{
		CString strTitle = _T ("ASL Serial Out Port Viewer - DISCONNECTED");
		AfxGetMainWnd ()->SetWindowText (strTitle);
		EnableDisable(false /*connected*/, false /*continuous reading*/);

		// Clear results table
		CListCtrl* list = (CListCtrl*)GetDlgItem(IDC_LIST1);
		list->DeleteAllItems();
		int nColumnCount = list->GetHeaderCtrl()->GetItemCount();
		if ( nColumnCount != 0 )
			for (int i=0; i<nColumnCount; i++)
			   list->DeleteColumn(0);

	}
	else
	{
		CString strTitle = _T ("ASL Serial Out Port Viewer - FAILED TO DISCONNECT");
		AfxGetMainWnd ()->SetWindowText (strTitle);
		CComBSTR bsError;
		gpISerialOutPort->GetLastError(&bsError);
		CString strError = bsError;
		MessageBox(strError, "Error disconnecting from serial port", MB_OK | MB_ICONERROR);
	}
}

void CETSerialPortViewerDlg::OnGetData() 
{
	LPSAFEARRAY items;
	long count;
	VARIANT_BOOL bAvailable;

	HRESULT hr;
	
	hr = gpISerialOutPort->GetScaledData(&items, &count, &bAvailable);

	if (FAILED(hr))
	{
		CComBSTR bsError;
		gpISerialOutPort->GetLastError(&bsError);
		CString strError = bsError;
		MessageBox(strError, "Error reading data", MB_OK | MB_ICONERROR);
		return;
	}

	if (bAvailable == VARIANT_TRUE)
	{
		// Display data record
		AddDataRecord(items, count);
		// Clean up
		SafeArrayDestroy(items);
	}

}

void CETSerialPortViewerDlg::OnStartContinuous() 
{
	HRESULT hr;
	// Create the sink object, and advise the server
	if (m_pHandler == NULL)
	{
		m_pHandler = new CEventHandler;
		m_pHandler->SetMainWindow(GetSafeHwnd());
		CComPtr<IUnknown> pUnk = gpISerialOutPort;
		hr = m_pHandler->DispEventAdvise(pUnk);
		ASSERT(SUCCEEDED(hr));
	}

	hr = gpISerialOutPort->StartContinuousMode();
	ASSERT(SUCCEEDED(hr));

	// Update user interface
	EnableDisable(true /*connected*/, true /*continuous reading*/);

	// Remember the mode (used in the OnClose() handler)
	m_bContinuous = true;
}

void CETSerialPortViewerDlg::OnStopContinuous() 
{
	if (gpISerialOutPort)
	{
		HRESULT hr = gpISerialOutPort->StopContinuousMode();
		ASSERT(SUCCEEDED(hr));
	}

	// Update user interface
	EnableDisable(true /*connected*/, false /*continuous reading*/);
	m_bContinuous = false;
}

void CETSerialPortViewerDlg::OnSelectCfgFile() 
{
	// Read serial port configuration from Eye Tracker file
    CString filename;
    GetDlgItem(IDC_CFG_FILE)->GetWindowText(filename);
    CFileDialog dlg(true, "cfg", filename, OFN_READONLY, "Eye Tracker Configuration Files|*.cfg");
    if(dlg.DoModal() == IDOK)
    {
        GetDlgItem(IDC_CFG_FILE)->SetWindowText(dlg.GetPathName());
    }
}

void CETSerialPortViewerDlg::OnClear() 
{
	// Clear the messages previously displayed
	CListCtrl* list = (CListCtrl*)GetDlgItem(IDC_LIST1);
	list->DeleteAllItems();
}

/////////////////////////////////////////////////////////////////////////////
// CETSerialPortViewerDlg private functions

// Update user interface
void CETSerialPortViewerDlg::EnableDisable (bool bConnected, bool bContinuousReading)
{
	// Entry boxes (should be disabled when connected)
	bool useCfg = (IsDlgButtonChecked(IDC_FORMAT_FROM_CFG_FILE) != 0);

	GetDlgItem(IDC_FORMAT_FROM_CFG_FILE)->EnableWindow(!bConnected);
	GetDlgItem(IDC_FORMAT_STANDARD)->EnableWindow(!bConnected);
	GetDlgItem(IDC_COM_PORT)->EnableWindow(!bConnected);
	GetDlgItem(IDC_EYEHEAD)->EnableWindow(!bConnected);

	GetDlgItem(IDC_CFG_FILE)->EnableWindow((!bConnected) && useCfg);
	GetDlgItem(IDC_SELECT_CFG_FILE)->EnableWindow(!bConnected && useCfg);

	GetDlgItem(IDC_UPDATE_RATE)->EnableWindow(!bConnected && !useCfg);
	GetDlgItem(IDC_STREAMING)->EnableWindow(!bConnected && !useCfg);


	// Buttons
    bool bStreaming =  (IsDlgButtonChecked(IDC_STREAMING) != 0);

	GetDlgItem(IDC_CONNECT)->EnableWindow(!bConnected);
	GetDlgItem(IDC_DISCONNECT)->EnableWindow(!bContinuousReading);
	GetDlgItem(IDC_GET_DATA)->EnableWindow(bConnected && (!bContinuousReading));
	GetDlgItem(IDC_START_CONTINUOUS)->EnableWindow(bConnected && bStreaming && (!bContinuousReading));
	GetDlgItem(IDC_STOP_CONTINUOUS)->EnableWindow(bConnected && bStreaming && bContinuousReading);
}

/////////////////////////////////////////////////////////////////////////////
// Message from callback handler


LRESULT CETSerialPortViewerDlg::OnNewMsg(WPARAM wParam, LPARAM lParam)
{
	LPSAFEARRAY* items = (LPSAFEARRAY*)wParam; 
	int count = lParam;

	// Display data record
	AddDataRecord(*items, count);

	// Clean up
	SafeArrayDestroy(*items);
	delete items;

	return (LRESULT)0;
}





void CETSerialPortViewerDlg::OnBnClickedWowstart()
{

  CWnd *pWndAct;
  CRect rect;

  // Open World of Warcraft window
  
  pWndAct = CWnd::FindWindow(NULL,"World of Warcraft");
  pWndAct->GetClientRect(&rect);
  pWndAct->SetForegroundWindow();
  

   AfxBeginThread(MyThreadProc, this);
}

void CETSerialPortViewerDlg::OnBnClickedBtest()
{
   EyeFixationTrigger* eft_left_mouse_click_trigger = new EyeFixationTrigger(100);
   eft_left_mouse_click_trigger->Add_Eye_Pos(1);
   eft_left_mouse_click_trigger->Add_Eye_Pos(20);
   eft_left_mouse_click_trigger->Add_Eye_Pos(3);
   eft_left_mouse_click_trigger->Add_Eye_Pos(50);
   eft_left_mouse_click_trigger->Add_Eye_Pos(15);
   eft_left_mouse_click_trigger->Add_Eye_Pos(100);

   delete eft_left_mouse_click_trigger;
}

void CETSerialPortViewerDlg::Display_Msg(CString str)
{
    listwowpercept = (CListCtrl*)GetDlgItem(IDC_WOWPLIST);
	listwowpercept->InsertItem (0, str, 0);
}

void CETSerialPortViewerDlg::OnBnClickedWowkill()
{
	// TODO: Add your control notification handler code here
	kill_wowthread = false;
}

void CETSerialPortViewerDlg::OnBnClickedClearwowstatus()
{
	// TODO: Add your control notification handler code here
    listwowpercept = (CListCtrl*)GetDlgItem(IDC_WOWPLIST);
	listwowpercept->DeleteAllItems();
}

void CETSerialPortViewerDlg::RetrieveXYPosData(LPSAFEARRAY data, long count, double& x_pos_dbl, double& y_pos_dbl)
{

	CComVariant value;
	// Retrieving the x and y position from the eye-tracker
    long i = 3;
	SafeArrayGetElement(data, &i, &value);
	value.ChangeType(VT_BSTR);
	CString str = value.bstrVal;
	x_pos_rep_str = str;
	CString strFinalVal;
	strFinalVal.Format("X: %s",str); 
//	Display_Msg(strFinalVal);
	x_pos_dbl = atof(x_pos_rep_str);
	VariantClear(&value); // prevent memory leak

    long p = 4;
	SafeArrayGetElement(data, &p, &value);
	value.ChangeType(VT_BSTR);
	str = value.bstrVal;
	y_pos_rep_str = str;
	strFinalVal.Format("Y: %s",str); 
//	Display_Msg(strFinalVal);
//	Display_Msg(str);
	y_pos_dbl = atof(y_pos_rep_str);
	VariantClear(&value); // prevent memory leak
}


CString CETSerialPortViewerDlg::Double_to_CString_Custom(double value)
{
	CString strValue,strInt, strDecimal;
	int decimal,sign;
	strValue = _fcvt(value,2,&decimal,&sign); 
    // Now decimal contains 1 because there is 
    // only one digit before the .

	strInt = strValue.Left(decimal); // strInt contains 4
	strDecimal = strValue.Mid(decimal); // strDecimal contains 125

	CString strFinalVal;
	strFinalVal.Format("%s.%s",strInt,strDecimal); 

	return strFinalVal;
}
void CETSerialPortViewerDlg::OnBnClickedButton1()
{
	// TODO: Add your control notification handler code here
	Display_Msg("----111----");
		// TODO: Add your control notification handler code here
	kill_wowthread = false;
}

void CETSerialPortViewerDlg::OnBnClickedButton2()
{
	// TODO: Add your control notification handler code here
	Display_Msg("----222----");
}
